"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Store = void 0;
const Path = __importStar(require("path"));
const path_oriented_data_structure_1 = require("@himenon/path-oriented-data-structure");
const dot_prop_1 = __importDefault(require("dot-prop"));
const Exception_1 = require("../../Exception");
const Def = __importStar(require("./Definition"));
const Operation = __importStar(require("./Operation"));
const State = __importStar(require("./State"));
const Structure = __importStar(require("./structure"));
class Store {
    constructor(factory, rootDocument) {
        this.factory = factory;
        this.state = State.createDefaultState(rootDocument);
        const { operator, getChildByPaths } = Structure.create();
        this.operator = operator;
        this.getChildByPaths = getChildByPaths;
    }
    convertNamespace(tree) {
        const statements = [];
        Object.values(tree.getChildren()).map(child => {
            if (child instanceof path_oriented_data_structure_1.Tree || child instanceof Structure.NamespaceTree.Item) {
                statements.push(this.convertNamespace(child));
            }
            else if (child instanceof Structure.InterfaceNode.Item) {
                statements.push(child.value);
            }
            else if (child instanceof Structure.TypeAliasNode.Item) {
                statements.push(child.value);
            }
        });
        if (tree instanceof Structure.NamespaceTree.Item) {
            return this.factory.Namespace.create({
                export: true,
                name: tree.params.name,
                statements,
                comment: tree.params.comment,
                deprecated: tree.params.deprecated,
            });
        }
        return this.factory.Namespace.create({
            export: true,
            name: tree.name,
            statements,
        });
    }
    capitalizeFirstLetter(text) {
        return text.charAt(0).toUpperCase() + text.slice(1);
    }
    getRootStatements() {
        // Debug Point: 抽象的なデータ構造全体を把握するために出力すると良い
        // fs.writeFileSync("debug/tree.json", JSON.stringify(this.operator.getHierarchy(), null, 2), { encoding: "utf-8" });
        const statements = Def.componentNames.reduce((statements, componentName) => {
            const treeOfNamespace = this.getChildByPaths(componentName, "namespace");
            if (treeOfNamespace) {
                treeOfNamespace.name = this.capitalizeFirstLetter(treeOfNamespace.name);
                return statements.concat(this.convertNamespace(treeOfNamespace));
            }
            return statements;
        }, []);
        return statements;
    }
    getAdditionalStatements() {
        return this.state.additionalStatements;
    }
    /**
     * @params path: "components/headers/hoge"
     */
    hasStatement(path, types) {
        const alreadyRegistered = types.some(type => !!this.operator.getChildByPaths(path, type));
        return alreadyRegistered;
    }
    /**
     * @params path: "components/headers/hoge"
     */
    addStatement(path, statement, options) {
        if (!path.startsWith("components")) {
            throw new Exception_1.UnSupportError(`componentsから始まっていません。path=${path}`);
        }
        const targetPath = Path.posix.relative("components", path);
        // すでにinterfaceまたはNAMESPACEとして登録がある場合はスキップ
        if (this.hasStatement(targetPath, ["interface", "namespace"])) {
            return;
        }
        // もしTypeAliasが同じスコープに登録されているかつ、interfaceが新しく追加しようとしている場合、既存のstatementを削除する
        if (!!(options === null || options === void 0 ? void 0 : options.override) || (this.hasStatement(targetPath, ["typeAlias"]) && statement.kind === "interface")) {
            this.operator.remove(targetPath, "typeAlias");
        }
        this.operator.set(targetPath, Structure.createInstance(statement));
    }
    getStatement(path, kind) {
        const targetPath = Path.posix.relative("components", path);
        // components/schemasの場合
        if (path.split("/").length === 2 && kind === "namespace") {
            const child = this.getChildByPaths(targetPath, kind);
            if (child) {
                // FIXME Side Effect
                child.name = this.capitalizeFirstLetter(child.name);
            }
            return child;
        }
        return this.getChildByPaths(targetPath, kind);
    }
    addComponent(componentName, statement) {
        this.operator.set(`${componentName}`, Structure.createInstance(statement));
    }
    getNoReferenceOperationState() {
        return Operation.create(this.state.document);
    }
    updateOperationState(httpMethod, requestUri, operationId, newOperationState) {
        let operationState = this.state.operations[operationId];
        if (operationState) {
            operationState = { ...operationState, ...newOperationState };
        }
        else {
            operationState = State.createDefaultOperationState(httpMethod, requestUri, newOperationState);
        }
        this.state.operations[operationId] = operationState;
    }
    addAdditionalStatement(statements) {
        this.state.additionalStatements = this.state.additionalStatements.concat(statements);
    }
    getPathItem(localPath) {
        if (!localPath.startsWith("components/pathItem")) {
            throw new Error("Only use start with 'component/pathItems': " + localPath);
        }
        const result = dot_prop_1.default.get(this.state.document, localPath.replace(/\//g, "."));
        if (!result) {
            throw new Error(`Not found ${localPath}`);
        }
        return result;
    }
    getParameter(localPath) {
        if (!localPath.startsWith("components/parameters")) {
            throw new Error("Only use start with 'component/parameters': " + localPath);
        }
        const result = dot_prop_1.default.get(this.state.document, localPath.replace(/\//g, "."));
        if (!result) {
            throw new Error(`Not found ${localPath}`);
        }
        return result;
    }
    isAfterDefined(referencePath) {
        return !!dot_prop_1.default.get(this.state.document, referencePath.replace(/\//g, "."));
    }
}
exports.Store = Store;
//# sourceMappingURL=Store.js.map