"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateStatements = exports.generateNamespace = void 0;
const os_1 = require("os");
const path = __importStar(require("path"));
const Guard = __importStar(require("../Guard"));
const Name = __importStar(require("../Name"));
const ExternalDocumentation = __importStar(require("./ExternalDocumentation"));
const Parameter = __importStar(require("./Parameter"));
const Reference = __importStar(require("./Reference"));
const RequestBody = __importStar(require("./RequestBody"));
const Responses = __importStar(require("./Responses"));
const Servers = __importStar(require("./Servers"));
const generateComment = (operation) => {
    const comments = [];
    if (operation.summary) {
        comments.push(operation.summary);
    }
    if (operation.description) {
        comments.push(operation.description);
    }
    if (operation.tags) {
        comments.push(`tags: ${operation.tags.join(", ")}`);
    }
    return comments.join(os_1.EOL);
};
const generateNamespace = (entryPoint, currentPoint, store, factory, parentPath, name, operation, pathItemParameters, context, converterContext) => {
    const basePath = `${parentPath}/${name}`;
    const operationId = operation.operationId;
    if (!operationId) {
        throw new Error("not setting operationId\n" + JSON.stringify(operation));
    }
    store.addStatement(basePath, {
        kind: "namespace",
        name,
        comment: ExternalDocumentation.addComment(Servers.addComment([generateComment(operation)], operation.servers), operation.externalDocs),
        deprecated: operation.deprecated,
    });
    const parameters = [...(pathItemParameters || []), ...(operation.parameters || [])];
    if (parameters.length > 0) {
        const parameterName = "Parameter";
        store.addStatement(`${basePath}/Parameter`, {
            kind: "interface",
            name: parameterName,
            value: Parameter.generateInterface(entryPoint, currentPoint, store, factory, parameterName, parameters, context, converterContext),
        });
    }
    if (operation.requestBody) {
        if (Guard.isReference(operation.requestBody)) {
            const reference = Reference.generate(entryPoint, currentPoint, operation.requestBody);
            if (reference.type === "local") {
                context.setReferenceHandler(currentPoint, reference);
                // TODO (not-use) 追加する必要がある（このメソッドを使わない可能性あり）
                factory.TypeReferenceNode.create({ name: context.resolveReferencePath(currentPoint, reference.path).name });
            }
            else if (reference.type === "remote" && reference.componentName) {
                const contentPath = path.join(reference.path, "Content"); // requestBodyはNamespaceを形成するため
                const name = "Content";
                store.addStatement(contentPath, {
                    kind: "interface",
                    name: name,
                    value: RequestBody.generateInterface(entryPoint, reference.referencePoint, factory, name, reference.data, context, converterContext),
                });
                const typeAliasName = context.resolveReferencePath(currentPoint, contentPath).name;
                store.addStatement(`${basePath}/RequestBody`, {
                    kind: "typeAlias",
                    name: typeAliasName,
                    value: factory.TypeAliasDeclaration.create({
                        export: true,
                        name: "RequestBody",
                        type: factory.TypeReferenceNode.create({ name: typeAliasName }),
                    }),
                });
            }
        }
        else {
            RequestBody.generateNamespace(entryPoint, currentPoint, store, factory, basePath, "RequestBody", operation.requestBody, context, converterContext);
        }
    }
    if (operation.responses) {
        Responses.generateNamespaceWithStatusCode(entryPoint, currentPoint, store, factory, basePath, operation.responses, context, converterContext);
    }
};
exports.generateNamespace = generateNamespace;
const generateStatements = (entryPoint, currentPoint, store, factory, requestUri, httpMethod, // PUT POST PATCH
operation, pathItemParameters, context, converterContext) => {
    let statements = [];
    const operationId = operation.operationId;
    if (!operationId) {
        throw new Error("not setting operationId\n" + JSON.stringify(operation));
    }
    store.updateOperationState(httpMethod, requestUri, operationId, {});
    const parameters = [...(pathItemParameters || []), ...(operation.parameters || [])];
    if (parameters.length > 0) {
        const parameterName = converterContext.generateParameterName(operationId);
        statements.push(Parameter.generateAliasInterface(entryPoint, currentPoint, store, factory, parameterName, parameters, context, converterContext));
    }
    if (operation.requestBody) {
        const requestBodyName = converterContext.generateRequestBodyName(operationId);
        if (Guard.isReference(operation.requestBody)) {
            const reference = Reference.generate(entryPoint, currentPoint, operation.requestBody);
            if (reference.type === "local") {
                context.setReferenceHandler(currentPoint, reference);
                statements.push(factory.TypeAliasDeclaration.create({
                    export: true,
                    name: converterContext.generateRequestBodyName(operationId),
                    type: factory.TypeReferenceNode.create({
                        name: context.resolveReferencePath(currentPoint, `${reference.path}`).name + "." + Name.ComponentChild.Content, // TODO Contextから作成？
                    }),
                }));
            }
            else if (reference.type === "remote" && reference.componentName) {
                const contentPath = path.join(reference.path, "Content"); // requestBodyはNamespaceを形成するため
                const name = "Content";
                store.addStatement(contentPath, {
                    kind: "interface",
                    name: name,
                    value: RequestBody.generateInterface(entryPoint, reference.referencePoint, factory, name, reference.data, context, converterContext),
                });
                statements.push(factory.TypeAliasDeclaration.create({
                    export: true,
                    name: converterContext.escapeDeclarationText(requestBodyName),
                    type: factory.TypeReferenceNode.create({ name: context.resolveReferencePath(currentPoint, contentPath).name }),
                }));
                store.updateOperationState(httpMethod, requestUri, operationId, {
                    requestBodyName: requestBodyName,
                });
            }
        }
        else {
            statements.push(RequestBody.generateInterface(entryPoint, currentPoint, factory, requestBodyName, operation.requestBody, context, converterContext));
            store.updateOperationState(httpMethod, requestUri, operationId, {
                requestBodyName: requestBodyName,
            });
        }
    }
    if (operation.responses) {
        statements = statements.concat(Responses.generateInterfacesWithStatusCode(entryPoint, currentPoint, store, factory, operationId, operation.responses, context, converterContext).flat());
    }
    return statements;
};
exports.generateStatements = generateStatements;
//# sourceMappingURL=Operation.js.map