import * as Utils from "../../utils";
/**
 * ユーザーが利用できる各種変換オプション
 */
export const create = () => {
    const convertReservedWord = (word) => {
        if (["import", "export"].includes(word)) {
            return word + "_";
        }
        return word;
    };
    const convertString = (text) => {
        if (Utils.isAvailableVariableName(text)) {
            return text;
        }
        return text.replace(/-/g, "$").replace(/\//g, "$");
    };
    return {
        escapeOperationIdText: (operationId) => {
            return convertString(operationId);
        },
        escapeDeclarationText: (text) => {
            // console.log(`escapeDeclarationText: ${text}` + `-> ${convertReservedWord(convertString(text).replace(/\./g, "$"))}`.padStart(100, " "));
            return convertReservedWord(convertString(text).replace(/\./g, "$"));
        },
        escapeReferenceDeclarationText: (text) => {
            // console.log(`escapeDeclarationText3: ${text}` + `-> ${convertReservedWord(convertString(text))}`.padStart(100, " "));
            return convertReservedWord(convertString(text));
        },
        escapePropertySignatureName: (text) => {
            return Utils.escapeText(text);
        },
        escapeTypeReferenceNodeName: (text) => {
            return convertString(text);
        },
        generateResponseName: (operationId, statusCode) => {
            return Utils.responseName(convertString(operationId), statusCode);
        },
        generateArgumentParamsTypeDeclaration: (operationId) => {
            return Utils.argumentParamsTypeDeclaration(convertString(operationId));
        },
        generateRequestContentTypeName: (operationId) => {
            return Utils.requestContentType(convertString(operationId));
        },
        generateResponseContentTypeName: (operationId) => {
            return Utils.responseContentType(convertString(operationId));
        },
        generateParameterName: (operationId) => {
            return Utils.parameterName(convertString(operationId));
        },
        generateRequestBodyName: (operationId) => {
            return Utils.requestBodyName(convertString(operationId));
        },
        generateFunctionName: (operationId) => {
            return convertString(operationId);
        },
    };
};
//# sourceMappingURL=ConverterContext.js.map