import ts from "typescript";
import { generateComment } from "./utils";
export const create = ({ factory }) => (params) => {
    const createNode = () => {
        if (typeof params.value === "string") {
            const literal = ts.setEmitFlags(factory.createStringLiteral(params.value), ts.EmitFlags.NoAsciiEscaping);
            return factory.createLiteralTypeNode(literal);
        }
        if (typeof params.value === "number") {
            return factory.createLiteralTypeNode(factory.createNumericLiteral(params.value));
        }
        return factory.createLiteralTypeNode(params.value ? factory.createTrue() : factory.createFalse());
    };
    const node = createNode();
    if (params.comment) {
        const comment = generateComment(params.comment);
        return ts.addSyntheticLeadingComment(node, ts.SyntaxKind.MultiLineCommentTrivia, comment.value, comment.hasTrailingNewLine);
    }
    return node;
};
export const make = (context) => {
    return {
        create: create(context),
    };
};
//# sourceMappingURL=LiteralTypeNode.js.map